/*
 * arginfo.c
 * 
 * Copyright (c) 2000, BindView Corporation.
 *
 * See LICENSE file.
 *
 */


#include "syscalls.h"

#define MagicFoo(call,args) struct arg_info arg_info_##call##_##args = { args/4 };
#include "syscallx.h"
#undef MagicFoo



struct arg_info *all_arg_info[] = {
#define MagicFoo(call,args) &arg_info_##call##_##args,
#include "syscallx.h"
#undef MagicFoo
};


enum all_types {
#define MagicFoo(type) typ_##type,
#include "typesx.h"
#undef MagicFoo
};


/*
 * Ok.  This is the disgustingly detailed stuff that can't be
 * solved with the preprocessor.  The type info for every syscall
 * needs to be specified, and there's no rhyme or reason to them.
 * The "default" (by virtue of what 0 means) for everything is to
 * be of type DWORD, with IN direction.  Anything other than that
 * needs to be specified here.
 *
 * Also, not every system call is fully understood, either.
 * (How's that for an understatement? :))
 */

/*
 * Hmmm, maybe the preprocessor _can_ help some. ;)
 */
#define set_arg(a,t,d) \
    all_arg_info[call]->args[a].type = t; \
    all_arg_info[call]->args[a].dir  = d;

void
init_arg_info (void)
{
#if 1
    int call;

    call = NtAcceptConnectPort_24;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (2, typ_PLPC_MESSAGE, DIR_IN_OUT);  /* FIXME: really just DIR_IN I think, but I'm curious */
    set_arg (5, typ_PLPC_SECTIONMAPINFO, DIR_IN_OUT);  /* FIXME: really just DIR_IN I think, but I'm curious */

    call = NtClose_4;
    set_arg (0, typ_HANDLE, DIR_IN);

    call = NtConnectPort_32;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_PUNICODE_STRING, DIR_IN);
    /* set_arg (2, typ_PSECURITY_QOS, DIR_IN); */
    set_arg (3, typ_PLPC_SECTIONINFO, DIR_IN_OUT);
    set_arg (4, typ_PLPC_SECTIONMAPINFO, DIR_IN_OUT);
    set_arg (5, typ_PHANDLE, DIR_OUT); /* actually, max msg len, not a handle */

    call = NtCreateDirectoryObject_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateEvent_20;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateEventPair_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateFile_44;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateKey_28;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);
    set_arg (4, typ_PUNICODE_STRING, DIR_IN);

    call = NtCreateMailslotFile_32;
    set_arg (0, typ_PHANDLE, DIR_OUT);

    call = NtCreateMutant_16;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateNamedPipeFile_56;
    set_arg (0, typ_PHANDLE, DIR_OUT);

    call = NtCreatePagingFile_16;
    set_arg (0, typ_PHANDLE, DIR_OUT);

    call = NtCreatePort_20;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateProcess_32;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateProfile_36;
    set_arg (0, typ_PHANDLE, DIR_OUT);

    call = NtCreateSection_28;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateSemaphore_20;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateSymbolicLinkObject_16;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateThread_32;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateTimer_16;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtCreateToken_52;
    set_arg (0, typ_PHANDLE, DIR_OUT);

    call = NtCreateWaitablePort_20;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_POBJ_ATTRIB, DIR_IN);

    call = NtDuplicateObject_28;
    set_arg (3, typ_PHANDLE, DIR_OUT);
    set_arg (4, typ_ACCESS_MASK, DIR_IN);

    call = NtImpersonateClientOfPort_8;
    set_arg (1, typ_PLPC_MESSAGE, DIR_IN);

    call = NtListenPort_8;
    set_arg (1, typ_PLPC_MESSAGE, DIR_OUT);

    call = NtOpenDirectoryObject_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenEvent_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenEventPair_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenFile_24;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenKey_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenMutant_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenProcess_16;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);
    set_arg (3, typ_PCLIENT_ID, DIR_IN);

    call = NtOpenProcessToken_12;
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_PHANDLE, DIR_OUT);

    call = NtOpenSection_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenSemaphore_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenSymbolicLinkObject_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtOpenThread_16;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);
    set_arg (3, typ_PCLIENT_ID, DIR_IN);

    call = NtOpenThreadToken_16;
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (3, typ_PHANDLE, DIR_OUT);

    call = NtOpenTimer_12;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_ACCESS_MASK, DIR_IN);
    set_arg (2, typ_POBJ_ATTRIB, DIR_IN);

    call = NtQueryAttributesFile_8;
    set_arg (0, typ_POBJ_ATTRIB, DIR_IN);

    call = NtQueryDirectoryObject_28;
    set_arg (5, typ_PHANDLE, DIR_IN_OUT);
    set_arg (6, typ_PHANDLE, DIR_OUT);

    call = NtQuerySymbolicLinkObject_12;
    set_arg (1, typ_PUNICODE_STRING, DIR_OUT);
    set_arg (2, typ_PHANDLE, DIR_OUT);

    call = NtQueryValueKey_24;
    set_arg (0, typ_HANDLE, DIR_IN);
    set_arg (1, typ_PUNICODE_STRING, DIR_IN);

    call = NtReadFile_36;
    set_arg (0, typ_HANDLE, DIR_IN);
    set_arg (1, typ_HANDLE, DIR_IN);

    call = NtReplyWaitReceivePort_16;
    set_arg (2, typ_PLPC_MESSAGE, DIR_IN);
    set_arg (3, typ_PLPC_MESSAGE, DIR_OUT);

    call = NtRequestWaitReplyPort_12;
    set_arg (1, typ_PLPC_MESSAGE, DIR_IN);
    set_arg (2, typ_PLPC_MESSAGE, DIR_OUT);
    
    call = NtSecureConnectPort_36;
    set_arg (0, typ_PHANDLE, DIR_OUT);
    set_arg (1, typ_PUNICODE_STRING, DIR_IN);

    call = NtSetValueKey_24;
    set_arg (0, typ_HANDLE, DIR_IN);
    set_arg (1, typ_PUNICODE_STRING, DIR_IN);
#endif
}
#undef set_arg
